@@ -12,7 +12,8 @@ Pod::2::DocBook - Convert Pod data to DocBook SGML
         doctype           => 'article',
         base_id           => 'article42'
         fix_double_quotes => 1,
-        spaces            => 3
+        spaces            => 3,
+        id_version        => 2,
     );
 
   $parser->parse_from_file ('my_article.pod', 'my_article.sgml');
@@ -83,7 +84,7 @@ use List::MoreUtils 'any';
 =cut
 
 use base 'Pod::Parser';
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 my $SPACE        = q{ };
 my $DOUBLE_QUOTE = q{"};
@@ -104,12 +105,17 @@ sub initialize {
     $parser->errorsub('error_msg');
     $parser->{'Pod::2::DocBook::errors'} = [];
 
-    $parser->{title}  ||= q{};
-    $parser->{spaces} ||= 0;
-   
+    $parser->{title}      ||= q{};
+    $parser->{spaces}     ||= 0;
+    $parser->{id_version} ||= 1;
+    my $skip = $parser->{skip};
+    $parser->{skip}       = [];
+    push @{$parser->{skip}}, split(/\s*,\s*/, $skip || '')
+        if $parser->{skip};
+    
     # if base_id not set, put title as base_id or a random number in worst case
     $parser->{base_id} ||= $parser->{title} || q{:}._big_random_number();
-    $parser->{base_id} = cleanup_id($parser->{base_id});
+    $parser->{base_id} = $parser->cleanup_id($parser->{base_id});
     
     return;
 }
@@ -202,6 +208,15 @@ sub command {
     my $out_fh = $parser->output_handle();
 
     return if $command eq 'pod';
+    
+    # check if we need to skip this heading
+    if ($command =~ /^head[1-4]/xms) {
+        $parser->{'skip_current'} = (
+            (any { $paragraph =~ m/^$_/  } @{$parser->{'skip'}})
+            ? 1
+            : 0
+        );
+    }
 
     $paragraph =~ s/\s+$//sx;
     $paragraph = $parser->interpolate($paragraph, $line_num);
@@ -229,6 +244,8 @@ sub command {
 
     if ($command =~ /^head[1-4]/xms) {
         $parser->_transition($command);
+        
+        return if $parser->{'skip_current'};
         $parser->_handle_head($command, $paragraph, $line_num);
     }
     elsif ($command eq 'begin') {
@@ -273,6 +290,8 @@ sub textblock {
     $state = q{} unless defined $state;
     $paragraph =~ s/\s+$//xms unless $state eq 'begin docbook';
 
+    $paragraph =~ s/&/&amp;/xmsg unless $state eq 'begin docbook';
+
     unless ($state eq 'begin docbook' || $state eq 'begin table') {
         $paragraph = $parser->interpolate($paragraph, $line_num);
         $paragraph = _fix_chars($paragraph);
@@ -383,7 +402,8 @@ sub textblock {
         }
     }
 
-    print $out_fh $para_out;
+    print $out_fh $para_out
+        if not $parser->{'skip_current'};
     
     return;
 }
@@ -401,6 +421,8 @@ sub verbatim {
     my @lines;
     my $min_leader;
 
+    return if $parser->{'skip_current'};
+
     $paragraph =~ s/\s+$//sx unless $state eq 'begin docbook';
 
     @lines = split(/\n/xms, $paragraph);
@@ -601,10 +623,19 @@ sub _current_indent {
 
 =head2 make_id($text)
 
+default id format -
+
 Function will construct an element id string. Id string is composed of
 C<< join (':', $parser->{base_id}, $text) >>, where C<$text> in most cases
 is the pod heading text.
 
+version 2 id format -
+
+having ':' in id was not a best choice. (Xerces complains - Attribute value
+"lib.Moose.Manual.pod:NAME" of type ID must be an NCName when namespaces are
+enabled.) To not break backwards compatibity switch with F<<id_version => 2>> in
+constructor for using '-' instead.
+
 The xml id string has strict format. Checkout L</"cleanup_id"> function for
 specification.
 
@@ -619,7 +650,10 @@ sub make_id {
     $text    =~ s/^\s*//xms;$text    =~ s/\s*$//xms;
     $base_id =~ s/^\s*//xms;$base_id =~ s/\s*$//xms;
     
-    return cleanup_id(join (':', $base_id, $text));
+    return $parser->cleanup_id(join ('-', $base_id, $text))
+        if $parser->{'id_version'} == 2;
+
+    return $parser->cleanup_id(join (':', $base_id, $text));
 }
 
 
@@ -674,6 +708,10 @@ sub _handle_L {
     # are de-protected in _fix_chars ()
 
     my ($text, $inferred, $name, $section, $type) = parselink($argument);
+    $inferred =~ s/&/&amp;/xmsg
+        if $inferred;
+    $name     =~ s/&/&amp;/xmsg
+        if $name;
 
     return qq!<ulink\37632\377url="$inferred">$inferred</ulink>!
       if $type eq 'url';
@@ -1163,6 +1201,7 @@ will be used.
 =cut
 
 sub cleanup_id {
+    my $parser    = shift;
     my $id_string = shift;
     
     $id_string =~ s/<!\[CDATA\[(.+?)\]\]>/$1/gxms;# keep just inside of CDATA
@@ -1170,7 +1209,10 @@ sub cleanup_id {
     $id_string =~ s/^\s*//xms;                    # ltrim spaces
     $id_string =~ s/\s*$//xms;                    # rtrim spaces
     $id_string =~ tr{/ }{._};                     # replace / with . and spaces with _
-    $id_string =~ s/[^\-_a-zA-Z0-9\.:\s]//gxms;   # closed set of characters allowed in id string
+    $id_string =~ s/[^\-_a-zA-Z0-9\.:]//gxms;     # closed set of characters allowed in id string
+    $id_string =~ s/^[^A-Za-z_:]+//xms;           # remove invalid leading characters
+    $id_string =~ s/:/_/xmsg                      # remove : in ids version 2
+        if $parser->{'id_version'} == 2;
 
     # check if the id string is valid (SEE http://www.w3.org/TR/2000/REC-xml-20001006#NT-Name)
     # TODO refactor to the function, we will need if also later and some tests will be handfull
@@ -1179,7 +1221,7 @@ sub cleanup_id {
         $id_string = q{:}._big_random_number();
         warn 'wrong xml id string "', $id_string, '", throwing away and using ', $id_string, ' instead!', "\n";
     }
-    
+
     return $id_string;
 }
 
